@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
-#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.9/rst2pdf/pygments_code_block_directive.py $
-#$LastChangedDate: 2008-09-03 17:36:52 -0300 (Wed, 03 Sep 2008) $
-#$LastChangedRevision: 265 $
+#$HeadURL: http://rst2pdf.googlecode.com/svn/tags/0.10.1/rst2pdf/pygments_code_block_directive.py $
+#$LastChangedDate: 2009-05-16 01:46:20 +0200 (Sa, 16. Mai 2009) $
+#$LastChangedRevision: 533 $
 
 # :Author: a Pygments author|contributor; Felix Wiemann; Guenter Milde
-# :Date: $Date: 2008-09-03 17:36:52 -0300 (Wed, 03 Sep 2008) $
+# :Date: $Date: 2009-05-16 01:46:20 +0200 (Sa, 16. Mai 2009) $
 # :Copyright: This module has been placed in the public domain.
-# 
+#
 # This is a merge of `Using Pygments in ReST documents`_ from the pygments_
 # documentation, and a `proof of concept`_ by Felix Wiemann.
-# 
+#
 # ========== ===========================================================
 # 2007-06-01 Removed redundancy from class values.
 # 2007-06-04 Merge of successive tokens of same type
@@ -19,21 +19,25 @@
 #            allowing the use of pygments-produced style sheets.
 # 2007-06-07 Merge in the formatting of the parsed tokens
 #            (misnamed as docutils_formatter) as class DocutilsInterface
-# 2007-06-08 Failsave implementation (fallback to a standard literal block 
+# 2007-06-08 Failsave implementation (fallback to a standard literal block
 #            if pygments not found)
 # ========== ===========================================================
-# 
+#
 # ::
 
-"""Define and register a code-block directive using pygments
-"""
+"""Define and register a code-block directive using pygments"""
+
 
 # Requirements
 # ------------
 # ::
 
+
+import codecs
+
 from docutils import nodes
 from docutils.parsers.rst import directives
+
 try:
     import pygments
     from pygments.lexers import get_lexer_by_name
@@ -41,38 +45,39 @@ try:
 except ImportError:
     pass
 
-import codecs
 from log import log
 
+
 # Customisation
 # -------------
-# 
+#
 # Do not insert inline nodes for the following tokens.
 # (You could add e.g. Token.Punctuation like ``['', 'p']``.) ::
 
 unstyled_tokens = ['']
 
+
 # DocutilsInterface
 # -----------------
-# 
+#
 # This interface class combines code from
 # pygments.formatters.html and pygments.formatters.others.
-# 
+#
 # It does not require anything of docutils and could also become a part of
 # pygments::
 
 class DocutilsInterface(object):
     """Parse `code` string and yield "classified" tokens.
-    
+
     Arguments
-    
+
       code     -- string of source code to parse
       language -- formal language the code is written in.
-    
-    Merge subsequent tokens of the same token-type. 
-    
-    Yields the tokens as ``(ttype_class, value)`` tuples, 
-    where ttype_class is taken from pygments.token.STANDARD_TYPES and 
+
+    Merge subsequent tokens of the same token-type.
+
+    Yields the tokens as ``(ttype_class, value)`` tuples,
+    where ttype_class is taken from pygments.token.STANDARD_TYPES and
     corresponds to the class argument used in pygments html output.
 
     """
@@ -80,7 +85,7 @@ class DocutilsInterface(object):
     def __init__(self, code, language):
         self.code = code
         self.language = language
-        
+
     def lex(self):
         # Get lexer for language (use text as fallback)
         try:
@@ -90,8 +95,8 @@ class DocutilsInterface(object):
             # what happens if pygment isn't present ?
             lexer = get_lexer_by_name('text')
         return pygments.lex(self.code, lexer)
-        
-            
+
+
     def join(self, tokens):
         """join subsequent tokens of same token-type
         """
@@ -112,7 +117,7 @@ class DocutilsInterface(object):
             tokens = self.lex()
         except IOError:
             log.info("Pygments lexer not found, using fallback")
-            # TODO: write message to INFO 
+            # TODO: write message to INFO
             yield ('', self.code)
             return
 
@@ -120,23 +125,21 @@ class DocutilsInterface(object):
             yield (_get_ttype_class(ttype), value)
 
 
-
 # code_block_directive
 # --------------------
 # ::
 
 def code_block_directive(name, arguments, options, content, lineno,
                        content_offset, block_text, state, state_machine):
-    """parse and classify content of a code_block
-    """
+    """Parse and classify content of a code_block."""
     if 'include' in options:
         try:
-            content=codecs.open(options['include'],'r','utf-8').read()
-        except IOError, UnicodeError: # no file or problem finding it or reading it
-            log.error('Error reading file: "%s" L %s' % (options['include'],lineno))
-            content=u''
+            content = codecs.open(options['include'], 'r', 'utf-8').read()
+        except (IOError, UnicodeError): # no file or problem finding it or reading it
+            log.error('Error reading file: "%s" L %s' % (options['include'], lineno))
+            content = u''
         if content:
-            # here we define the start-at and end-at options 
+            # here we define the start-at and end-at options
             # so that limit is included in extraction
             # this is different than the start-after directive of docutils
             # (docutils/parsers/rst/directives/misc.py L73+)
@@ -153,18 +156,18 @@ def code_block_directive(name, arguments, options, content, lineno,
                     raise state_machine.reporter.severe('Problem with "start-at" option of "%s" '
                                       'code-block directive:\nText not found.' % options['start-at'])
                 content = content[after_index:]
-                
+
             after_text = options.get('start-after', None)
             if after_text:
                 # skip content in include_text before *and incl.* a matching text
-                print type(content),type(after_text)
+                print type(content), type(after_text)
                 after_index = content.find(after_text)
                 if after_index < 0:
                     raise state_machine.reporter.severe('Problem with "start-after" option of "%s" '
                                       'code-block directive:\nText not found.' % options['start-after'])
                 content = content[after_index + len(after_text):]
-                
-                
+
+
             # same changes here for the same reason
             before_text = options.get('end-at', None)
             if before_text:
@@ -174,7 +177,7 @@ def code_block_directive(name, arguments, options, content, lineno,
                     raise state_machine.reporter.severe('Problem with "end-at" option of "%s" '
                                       'code-block directive:\nText not found.' % options['end-at'])
                 content = content[:before_index + len(before_text)]
-                
+
             before_text = options.get('end-before', None)
             if before_text:
                 # skip content in include_text after *and NOT incl.* a matching text
@@ -185,19 +188,41 @@ def code_block_directive(name, arguments, options, content, lineno,
                 content = content[:before_index]
 
     else:
-        content=u'\n'.join(content)
+        content = u'\n'.join(content)
+
+    withln = "linenos" in options
 
     language = arguments[0]
     # create a literal block element and set class argument
     code_block = nodes.literal_block(classes=["code", language])
-    
+
+    if withln:
+        lineno = 1
+        total_lines = content.count('\n') + 1
+        lnwidth = len(str(total_lines))
+        fstr = "\n%%%dd " % lnwidth
+        code_block += nodes.Text(fstr[1:] % lineno, fstr[1:] % lineno)
+
     # parse content with pygments and add to code_block element
     for cls, value in DocutilsInterface(content, language):
-        if cls in unstyled_tokens:
+        if withln and "\n" in value:
+            # Split on the "\n"s
+            values = value.split("\n")
+            # The first piece, pass as-is
+            code_block += nodes.Text(values[0], values[0])
+            # On the second and later pieces, insert \n and linenos
+            linenos = range(lineno, lineno + len(values))
+            for chunk, ln in zip(values, linenos)[1:]:
+                if ln <= total_lines:
+                    code_block += nodes.Text(fstr % ln, fstr % ln)
+                    code_block += nodes.Text(chunk, chunk)
+            lineno += len(values) - 1
+
+        elif cls in unstyled_tokens:
             # insert as Text to decrease the verbosity of the output.
             code_block += nodes.Text(value, value)
         else:
-            code_block += nodes.inline(value, value, classes=["pygments-"+cls])
+            code_block += nodes.inline(value, value, classes=["pygments-" + cls])
 
     return [code_block]
 
@@ -213,19 +238,21 @@ code_block_directive.options = { 'include' : directives.unchanged_required,
                                  'end-at' : directives.unchanged_required,
                                  'start-after' : directives.unchanged_required,
                                  'end-before' : directives.unchanged_required,
+                                 'linenos' : directives.unchanged,
                                  }
 
 directives.register_directive('code-block', code_block_directive)
 
+
 # .. _doctutils: http://docutils.sf.net/
 # .. _pygments: http://pygments.org/
 # .. _Using Pygments in ReST documents: http://pygments.org/docs/rstdirective/
 # .. _proof of concept:
 #      http://article.gmane.org/gmane.text.docutils.user/3689
-# 
+#
 # Test output
 # -----------
-# 
+#
 # If called from the command line, call the docutils publisher to render the
 # input::
 
@@ -235,9 +262,6 @@ if __name__ == '__main__':
     try:
         import locale
         locale.setlocale(locale.LC_ALL, '')
-    except:
+    except Exception:
         pass
     publish_cmdline(writer_name='pdf', description=description)
-    
-
-
